<?php

namespace App\Http\Controllers;

use App\AdminOtherAccount;
use App\AdminSalesInvoice;
use App\AdminWarehouse;
use App\Company;
use App\Customer;
use App\DebitNote;
use App\DebitNoteExpense;
use App\Expense;
use App\ItemName;
use App\Quotation;
use App\StaffAndSalesman;
use App\Supplier;
use App\WareHouse;
use App\AdminPurchaesInvoice;
use App\OtherAccountBuyer;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class DebitNoteController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $debitNotes = DebitNote::with('expenses')
            ->whereIn('id', function ($query) {
                $query->selectRaw('MIN(id)')
                    ->from('debit_notes')
                    ->where('branch_id', session()->get('selected_branch'))
                    ->whereNotNull('invoice_no')
                    ->groupBy('invoice_no');
            })
            ->withTrashed()
            ->orderBy('id', 'desc')->get();

        return view('crusher.trans_debit_note_prv.index', compact('debitNotes'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $customers = Customer::where('branch_id', session()->get('selected_branch'))->get();
        $items = ItemName::where('branch_id', session()->get('selected_branch'))->get();
        $warehouses = AdminWareHouse::where('branch_id', session()->get('selected_branch'))->get();
        $expenses = Expense::where(['branch_id'=>session()->get('selected_branch'), 'class_of_account_id'=>'Tax & Duties'])
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\Expense');
        });
        $otherAccountExpenses = AdminOtherAccount::where(['branch_id'=>session()->get('selected_branch'), 'class_of_account'=>'Tax & Duties'])
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\AdminOtherAccount');
        });

        $expensesAccount = $expenses->concat($otherAccountExpenses);

        $currentDate = Carbon::now()->format('Ymd');

        // Get the latest quotation number for the branch
        $latestCn = DebitNote::where('branch_id', session()->get('selected_branch'))
            ->latest('id')->first();

        if ($latestCn) {
            $lastNumber = (int)substr($latestCn->invoice_no, -4); // Extract the last 4 digits
            $newNumber = $lastNumber + 1;
            $newCn = str_pad($newNumber, 4, '0', STR_PAD_LEFT);
        } else {
            $newCn = '0001';
        }



        $debitNo = request()->input('pi_no');

        if ($debitNo) {
            $debits = AdminPurchaesInvoice::with('account', 'item', 'warehouse',)->where('invoice_no', $debitNo)->get();
        } else {
            $debits = [];
        }


        $adminSales = AdminSalesInvoice::with('account', 'item')
            ->whereIn('id', function ($query) {
                $query->selectRaw('MIN(id)')
                    ->from('admin_sales_invoices')
                    ->groupBy('invoice_no');
            })->where('branch_id', session()->get('selected_branch'))
            ->get();

        // Retrieve customers and include both 'account_name' and 'table_name' attributes
        $customers = Customer::where('branch_id', session()->get('selected_branch'))
        ->select('*', 'name_of_organisation as account_name')
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\Customer');
        });

        // Retrieve suppliers and include both 'account_name' and 'table_name' attributes
        $suppliers = Supplier::where('branch_id', session()->get('selected_branch'))
        ->select('*', 'name_of_organisation as account_name')
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\Supplier');
        });

        // Retrieve expenses and set 'table_name' to the fully-qualified model class name
                $salesmens = StaffAndSalesman::where('branch_id', session()->get('selected_branch'))
        ->select('*', DB::raw("CONCAT(first_name, ' ', last_name) as account_name"))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\StaffAndSalesman');
        });

        // Retrieve expenses and set 'table_name' to the fully-qualified model class name
        $expenses = Expense::where('branch_id', session()->get('selected_branch'))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\Expense');
        });

        // Retrieve other accounts and set 'table_name' to the fully-qualified model class name
        $otherAccounts = AdminOtherAccount::where('branch_id', session()->get('selected_branch'))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\AdminOtherAccount');
        });

        // Concatenate all collections into a single collection
        $accounts = $customers->concat($expenses)->concat($suppliers)->concat($otherAccounts)->concat($salesmens);

        // dd($expensesAccount);
        return view('crusher.trans_debit_note_prv.create', compact('customers', 'items', 'newCn', 'warehouses', 'expensesAccount', 'adminSales', 'accounts','debits'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        // dd($request->all());
        try {
            $validatedData = $request->validate([
                'date' => 'nullable|date',
                'invoice_no' => 'nullable|string',
                'sales_invoice_no' => 'nullable|exists:admin_sales_invoices,invoice_no',
                'account_id' => 'nullable',
                'account_type' => 'nullable',
                'item_description' => 'nullable|array',
                'item_description.*' => 'nullable|string',
                'quantity' => 'nullable|array',
                'quantity.*' => 'nullable|numeric',
                'rate_unit' => 'nullable|array',
                'rate_unit.*' => 'nullable|numeric',
                'gross_amount' => 'nullable|array',
                'gross_amount.*' => 'nullable|numeric',
                'vat_percentage' => 'nullable|array',
                'vat_percentage.*' => 'nullable|numeric',
                'vat_amount' => 'nullable|array',
                'vat_amount.*' => 'nullable|numeric',
                'net_amount' => 'nullable|array',
                'net_amount.*' => 'nullable|numeric',
                'expense_type' => 'nullable|array',
                'expense_type.*' => 'nullable',
                'expense_id' => 'nullable|array',
                'expense_id.*' => 'nullable',
                'item_id' => 'nullable|array',
                'item_id.*' => 'nullable|exists:item_names,id',
                'warehouse_id' => 'nullable|array',
                'warehouse_id.*' => 'nullable|exists:admin_warehouses,id',
                'expense_amount' => 'nullable|array',
                'expense_amount.*' => 'nullable|numeric',
                'total_quantity' => 'nullable|numeric',
                'total_gross_amount' => 'nullable|numeric',
                'total_vat_amount' => 'nullable|numeric',
                'total_net_amount' => 'nullable|numeric',
                'AmountInWords' => 'nullable|string',
                'narrations' => 'nullable',
                'name' => 'nullable|string',
                'phone' => 'nullable|string',
                'email' => 'nullable|string',
                'vat_number' => 'nullable|string',
                'address' => 'nullable|string',
                'type' => 'nullable|string',
            ]);
            $branch_id = session()->get('selected_branch');

            // Iterate over the item descriptions
            for ($i = 0; $i < count($validatedData['quantity']); $i++) {
                $debitNoteData = [
                    'sales_invoice_no' => $validatedData['sales_invoice_no'],
                    'invoice_no' => $validatedData['invoice_no'],
                    'date' => $validatedData['date'],
                    'account_id' => $validatedData['account_id'],
                    'account_type' => $validatedData['account_type'],
                    'item_description' => $validatedData['item_description'][$i] ?? (NULL),
                    'quantity' => $validatedData['quantity'][$i] ?? (NULL),
                    'rate_unit' => $validatedData['rate_unit'][$i] ?? (NULL),
                    'gross_amount' => $validatedData['gross_amount'][$i] ?? (NULL),
                    'vat_percentage' => $validatedData['vat_percentage'][$i] ?? (NULL),
                    'vat_amount' => $validatedData['vat_amount'][$i] ?? (NULL),
                    'net_amount' => $validatedData['net_amount'][$i] ?? (NULL),
                    'item_id' => $validatedData['item_id'][$i] ?? (NULL),
                    'warehouse_id' => $validatedData['warehouse_id'][$i] ?? (NULL),
                    'total_quantity' => $validatedData['total_quantity'],
                    'total_gross_amount' => $validatedData['total_gross_amount'],
                    'total_vat_amount' => $validatedData['total_vat_amount'],
                    'total_net_amount' => $validatedData['total_net_amount'],
                    'AmountInWords' => $validatedData['AmountInWords'],
                    'narrations' => $validatedData['narrations'],
                    'branch_id' => $branch_id,
                ];

                $debitNote = DebitNote::create($debitNoteData);

                // Add expenses for this creditNote
                // for ($j = 0; $j < count($validatedData['expense_id']); $j++) {
                //     DebitNoteExpense::create([
                //         'debit_note_id' => $debitNote->id,
                //         'expense_type' => $validatedData['expense_type'][$j],
                //         'expense_id' => $validatedData['expense_id'][$j],
                //         'expense_amount' => $validatedData['expense_amount'][$j],
                //     ]);
                // }
            }

            flash()->addSuccess('Record Created');
            return redirect()->route('crusher.debit_notes.index');
        } catch (\Throwable $th) {
            // dd($th);
            flash()->addError($th->getMessage());
            return redirect()->back()->withInput();
        }
    }

    public function show($debitNote)
    {
        $debitNote = DebitNote::withTrashed()->find($debitNote);
        $debitNote = $debitNote->withTrashed()->with('account', 'item', 'warehouse', 'expenses.expense')->where('invoice_no', $debitNote->invoice_no)->get();
        $company = Company::where('user_id', auth()->id())->first();
        return view('crusher.trans_debit_note_prv.view', compact('debitNote', 'company'));
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\Quotation  $quotation
     * @return \Illuminate\Http\Response
     */
    public function edit(DebitNote $debitNote)
    {
        $customers = Customer::where('branch_id', session()->get('selected_branch'))->get();
        $items = ItemName::where('branch_id', session()->get('selected_branch'))->get();
        $warehouses = AdminWarehouse::where('branch_id', session()->get('selected_branch'))->get();
        $expenses = Expense::where(['branch_id'=>session()->get('selected_branch'), 'class_of_account_id'=>'Tax & Duties'])
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\Expense');
        });
        $otherAccountExpenses = AdminOtherAccount::where(['branch_id'=>session()->get('selected_branch'), 'class_of_account'=>'Tax & Duties'])
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\AdminOtherAccount');
        });

        $expensesAccount = $expenses->concat($otherAccountExpenses);

        $adminSales = AdminSalesInvoice::with('account', 'item')
            ->whereIn('id', function ($query) {
                $query->selectRaw('MIN(id)')
                    ->from('admin_sales_invoices')
                    ->groupBy('invoice_no');
            })->where('branch_id', session()->get('selected_branch'))
            ->get();

        // Retrieve customers and include both 'account_name' and 'table_name' attributes
        $customers = Customer::where('branch_id', session()->get('selected_branch'))
        ->select('*', 'name_of_organisation as account_name')
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\Customer');
        });

        // Retrieve suppliers and include both 'account_name' and 'table_name' attributes
        $suppliers = Supplier::where('branch_id', session()->get('selected_branch'))
        ->select('*', 'name_of_organisation as account_name')
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\Supplier');
        });

        // Retrieve expenses and set 'table_name' to the fully-qualified model class name
                $salesmens = StaffAndSalesman::where('branch_id', session()->get('selected_branch'))
        ->select('*', DB::raw("CONCAT(first_name, ' ', last_name) as account_name"))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\StaffAndSalesman');
        });

        // Retrieve expenses and set 'table_name' to the fully-qualified model class name
        $expenses = Expense::where('branch_id', session()->get('selected_branch'))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\Expense');
        });

        // Retrieve other accounts and set 'table_name' to the fully-qualified model class name
        $otherAccounts = AdminOtherAccount::where('branch_id', session()->get('selected_branch'))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\AdminOtherAccount');
        });

        // Concatenate all collections into a single collection
        $accounts = $customers->concat($expenses)->concat($suppliers)->concat($otherAccounts)->concat($salesmens);


        // $debitNotes = $debitNote->with('account', 'item', 'warehouse', 'expenses')->where('invoice_no', $debitNote->invoice_no)->get();

        $debitNotes = DebitNote::where('branch_id', session()->get('selected_branch'))
        ->with(['account', 'item', 'warehouse', 'expenses'])
        ->where('invoice_no', $debitNote->invoice_no)
        ->get();


        // dd($debitNotes);

        return view('crusher.trans_debit_note_prv.edit', compact('customers', 'items', 'warehouses', 'expensesAccount', 'adminSales', 'accounts', 'debitNotes'));
    }

    public function update(Request $request, DebitNote $debitNote)
    {
        try {
            $validatedData = $request->validate([
                'date' => 'nullable|date',
                'invoice_no' => 'nullable|string',
                'sales_invoice_no' => 'nullable|exists:admin_sales_invoices,invoice_no',
                'account_id' => 'nullable',
                'account_type' => 'nullable',
                'item_description' => 'nullable|array',
                'item_description.*' => 'nullable|string',
                'quantity' => 'nullable|array',
                'quantity.*' => 'nullable|numeric',
                'rate_unit' => 'nullable|array',
                'rate_unit.*' => 'nullable|numeric',
                'gross_amount' => 'nullable|array',
                'gross_amount.*' => 'nullable|numeric',
                'vat_percentage' => 'nullable|array',
                'vat_percentage.*' => 'nullable|numeric',
                'vat_amount' => 'nullable|array',
                'vat_amount.*' => 'nullable|numeric',
                'net_amount' => 'nullable|array',
                'net_amount.*' => 'nullable|numeric',
                'expense_type' => 'nullable|array',
                'expense_type.*' => 'nullable',
                'expense_id' => 'nullable|array',
                'expense_id.*' => 'nullable',
                'item_id' => 'nullable|array',
                'item_id.*' => 'nullable|exists:item_names,id',
                'warehouse_id' => 'nullable|array',
                'warehouse_id.*' => 'nullable|exists:admin_warehouses,id',
                'expense_amount' => 'nullable|array',
                'expense_amount.*' => 'nullable|numeric',
                'total_quantity' => 'nullable|numeric',
                'total_gross_amount' => 'nullable|numeric',
                'total_vat_amount' => 'nullable|numeric',
                'total_net_amount' => 'nullable|numeric',
                'AmountInWords' => 'nullable|string',
                'narrations' => 'nullable',
            ]);
            $branch_id = session()->get('selected_branch');
            $debitNote = $debitNote->with('account', 'item', 'warehouse', 'expenses')->where('invoice_no', $debitNote->invoice_no)->forceDelete();

            // Iterate over the item descriptions
            for ($i = 0; $i < count($validatedData['quantity']); $i++) {
                $debitNoteData = [
                    'sales_invoice_no' => $validatedData['sales_invoice_no'],
                    'invoice_no' => $validatedData['invoice_no'],
                    'date' => $validatedData['date'],
                    'account_id' => $validatedData['account_id'],
                    'account_type' => $validatedData['account_type'],
                    'item_description' => $validatedData['item_description'][$i] ?? (NULL),
                    'quantity' => $validatedData['quantity'][$i] ?? (NULL),
                    'rate_unit' => $validatedData['rate_unit'][$i] ?? (NULL),
                    'gross_amount' => $validatedData['gross_amount'][$i] ?? (NULL),
                    'vat_percentage' => $validatedData['vat_percentage'][$i] ?? (NULL),
                    'vat_amount' => $validatedData['vat_amount'][$i] ?? (NULL),
                    'net_amount' => $validatedData['net_amount'][$i] ?? (NULL),
                    'item_id' => $validatedData['item_id'][$i] ?? (NULL),
                    'warehouse_id' => $validatedData['warehouse_id'][$i] ?? (NULL),
                    'total_quantity' => $validatedData['total_quantity'],
                    'total_gross_amount' => $validatedData['total_gross_amount'],
                    'total_vat_amount' => $validatedData['total_vat_amount'],
                    'total_net_amount' => $validatedData['total_net_amount'],
                    'AmountInWords' => $validatedData['AmountInWords'],
                    'narrations' => $validatedData['narrations'],
                    'branch_id' => $branch_id,
                ];


                $debitNote = DebitNote::create($debitNoteData);

                // DebitNoteExpense::where('debit_note_id', $debitNote->id)->forceDelete();
                // // Add expenses for this creditNote
                // for ($j = 0; $j < count($validatedData['expense_id']); $j++) {
                //     DebitNoteExpense::create([
                //         'debit_note_id' => $debitNote->id,
                //         'expense_type' => $validatedData['expense_type'][$j],
                //         'expense_id' => $validatedData['expense_id'][$j],
                //         'expense_amount' => $validatedData['expense_amount'][$j],
                //     ]);
                // }
            }

            flash()->addSuccess('Record Update');
            return redirect()->route('crusher.debit_notes.index');
        } catch (\Throwable $th) {
            flash()->addError($th->getMessage());
            return redirect()->back()->withInput();
        }
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\Quotation  $quotation
     * @return \Illuminate\Http\Response
     */
    public function destroy(Quotation $quotation)
    {
        //
    }

    public function softDelete()
    {
        $debitNotes = DebitNote::where('invoice_no', request()->invoice_no)->get();

        if ($debitNotes->isEmpty()) {
            flash()->addError('No records found for the given invoice number.');
            return redirect()->route('crusher.debit_notes.index');
        }

        foreach ($debitNotes as $debitNote) {
            $debitNote->delete();
        }

        flash()->addSuccess('Success');
        return redirect()->route('crusher.debit_notes.index');
    }
}
