<?php

namespace App\Http\Controllers;

use App\Expense;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Http\Request;
use App\Http\Requests\StoreExpensesRequest;
use App\Http\Requests\UpdateExpensesRequest;

class ExpensesMasterController extends Controller
{
    use SoftDeletes;

    public function index()
    {
        $expenses = Expense::where('branch_id', session()->get('selected_branch'))->orderBy('id', 'desc')
        ->get();
        return view('crusher.expenses_master.index', compact('expenses'));
    }

    public function create()
    {
        return view('crusher.expenses_master.create');
    }

    function store(StoreExpensesRequest $request)
    {
        try {
            $validatedData = $request->validated();
            $validatedData['branch_id'] = session()->get('selected_branch');
            Expense::create($validatedData);

            flash()->addSuccess('Record Created');
            return redirect()->route('crusher.expense.index');
        } catch (\Throwable $th) {
            flash()->addError($th->getMessage());
            return redirect()->back()->withInput();
        }
    }

    public function edit($expense)
    {
        $expense = Expense::find($expense);
        return view('crusher.expenses_master.edit', compact('expense'));
    }

    function update(UpdateExpensesRequest $request, $expense)
    {
        try {
            $validatedData = $request->validated();
            $validatedData['branch_id'] = session()->get('selected_branch');

            Expense::find($expense)->update($validatedData);

            flash()->addSuccess('Record Updated');
            return redirect()->route('crusher.expense.index');
        } catch (\Throwable $th) {
            flash()->addError($th->getMessage());
            return redirect()->back()->withInput();
        }
    }

    public function show($expense)
    {
        $expense = Expense::find($expense);
        return view('crusher.expenses_master.view', compact('expense'));
    }

    public function softDelete($expense)
    {
        $expense = Expense::findOrFail($expense);
        $expense->delete();
        flash()->addSuccess('Success');
        return redirect()->route('crusher.expense.index');
    }
}
