<?php

namespace App\Http\Controllers;

use App\AdminOtherAccount;
use App\AdminTermsAndCondition;
use App\Company;
use App\Customer;
use App\Expense;
use App\OtherAccountBuyer;
use App\Quotation;
use App\QuotationExpense;
use App\StaffAndSalesman;
use App\Supplier;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class QuotationController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $quotations = Quotation::with('expenses', 'account')
        ->whereIn('id', function ($query) {
            $query->selectRaw('MIN(id)')
                ->from('quotations')
                ->groupBy('quotation_no');
        })
        ->where('branch_id', session()->get('selected_branch'))
        ->withTrashed()
        ->orderBy('id', 'desc')->get();



        return view('crusher.trans_quotation.index', compact('quotations'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $customers = Customer::where('branch_id', session()->get('selected_branch'))->get();
        $expenses = Expense::where(['branch_id'=>session()->get('selected_branch'), 'class_of_account_id'=>'Tax & Duties'])
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\Expense');
        });
        $otherAccountExpenses = AdminOtherAccount::where(['branch_id'=>session()->get('selected_branch'), 'class_of_account'=>'Tax & Duties'])
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\AdminOtherAccount');
        });

        $expensesAccount = $expenses->concat($otherAccountExpenses);
        $currentDate = Carbon::now()->format('Ymd');

        // Get the latest quotation number for the branch
        $latestQuotation = Quotation::where('branch_id', session()->get('selected_branch'))
            ->latest('id')->first();

       // Calculate the new quotation number
        if ($latestQuotation) {
            $explodedQuotationNo = explode('-', $latestQuotation->quotation_no);
            $lastQuotationNumber = end($explodedQuotationNo);
            $newQuotationNumber = $currentDate . '-' . str_pad((int)$lastQuotationNumber + 1, 4, '0', STR_PAD_LEFT);
        } else {
            $newQuotationNumber = $currentDate . '-0001';
        }


        // Retrieve customers and include both 'account_name' and 'table_name' attributes
        $customers = Customer::where('branch_id', session()->get('selected_branch'))
        ->select('*', 'name_of_organisation as account_name')
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\Customer');
        });

        // Retrieve suppliers and include both 'account_name' and 'table_name' attributes
        $suppliers = Supplier::where('branch_id', session()->get('selected_branch'))
        ->select('*', 'name_of_organisation as account_name')
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\Supplier');
        });

        // Retrieve expenses and set 'table_name' to the fully-qualified model class name
                $salesmens = StaffAndSalesman::where('branch_id', session()->get('selected_branch'))
        ->select('*', DB::raw("CONCAT(first_name, ' ', last_name) as account_name"))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\StaffAndSalesman');
        });

        // Retrieve expenses and set 'table_name' to the fully-qualified model class name
        $expenses = Expense::where('branch_id', session()->get('selected_branch'))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\Expense');
        });

        // Retrieve other accounts and set 'table_name' to the fully-qualified model class name
        $otherAccounts = AdminOtherAccount::where('branch_id', session()->get('selected_branch'))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\AdminOtherAccount');
        });

        // Concatenate all collections into a single collection
        $accounts = $customers->concat($expenses)->concat($suppliers)->concat($otherAccounts)->concat($salesmens);

        return view('crusher.trans_quotation.create', compact('customers', 'expensesAccount', 'newQuotationNumber', 'accounts'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        try {
            $validatedData = $request->validate([
                'quotation_no' => 'required|string',
                'date' => 'required|date',
                'account_id' => 'required',
                'account_type' => 'required',
                'item_description' => 'required|array',
                'item_description.*' => 'required|string',
                'quantity' => 'required|array',
                'quantity.*' => 'required|numeric',
                'rate_unit' => 'required|array',
                'rate_unit.*' => 'required|numeric',
                'gross_amount' => 'required|array',
                'gross_amount.*' => 'required|numeric',
                'vat_percentage' => 'required|array',
                'vat_percentage.*' => 'required|numeric',
                'vat_amount' => 'required|array',
                'vat_amount.*' => 'required|numeric',
                'net_amount' => 'required|array',
                'net_amount.*' => 'required|numeric',
                'expense_type' => 'nullable|array',
                'expense_type.*' => 'nullable',
                'expense_id' => 'nullable|array',
                'expense_id.*' => 'nullable',
                'expense_amount' => 'nullable|array',
                'expense_amount.*' => 'nullable|numeric',
                'total_quantity' => 'required|numeric',
                'total_gross_amount' => 'required|numeric',
                'total_vat_amount' => 'required|numeric',
                'total_net_amount' => 'required|numeric',
                'discount' => 'nullable|numeric',
                'name' => 'nullable|string',
                'phone' => 'nullable|string',
                'email' => 'nullable|string',
                'vat_number' => 'nullable|string',
                'address' => 'nullable|string',
                'type' => 'nullable|string',
                'invoice_no' => 'nullable|string',
            ]);
            $branch_id = session()->get('selected_branch');

            // Iterate over the item descriptions
            for ($i = 0; $i < count($validatedData['quantity']); $i++)
            {
                $quotationData = [
                    'quotation_no' => $validatedData['quotation_no'],
                    'date' => $validatedData['date'],
                    'account_id' => $validatedData['account_id'],
                    'account_type' => $validatedData['account_type'],
                    'item_description' => $validatedData['item_description'][$i]??(NULL),
                    'quantity' => $validatedData['quantity'][$i]??(NULL),
                    'rate_unit' => $validatedData['rate_unit'][$i]??(NULL),
                    'gross_amount' => $validatedData['gross_amount'][$i]??(NULL),
                    'vat_percentage' => $validatedData['vat_percentage'][$i]??(NULL),
                    'vat_amount' => $validatedData['vat_amount'][$i]??(NULL),
                    'net_amount' => $validatedData['net_amount'][$i]??(NULL),
                    'total_quantity' => $validatedData['total_quantity'],
                    'total_gross_amount' => $validatedData['total_gross_amount'],
                    'total_vat_amount' => $validatedData['total_vat_amount'],
                    'total_net_amount' => $validatedData['total_net_amount'],
                    'discount' => $validatedData['discount'],
                    'branch_id' => $branch_id,
                ];

                $quotation = Quotation::create($quotationData);

                // Add expenses for this quotation
                // for ($j = 0; $j < count($validatedData['expense_id']); $j++) {
                //     QuotationExpense::create([
                //         'quotation_id' => $quotation->id,
                //         'expense_type' => $validatedData['expense_type'][$j],
                //         'expense_id' => $validatedData['expense_id'][$j],
                //         'expense_amount' => $validatedData['expense_amount'][$j],
                //     ]);
                // }
            }

            flash()->addSuccess('Record Created');
            return redirect()->route('crusher.quotations.index');
        } catch (\Throwable $th) {
            flash()->addError($th->getMessage());
            return redirect()->back()->withInput();
        }
    }

    public function show($quotation)
    {
        $quotation = Quotation::withTrashed()->find($quotation);
        $quotation = $quotation->withTrashed()->with('account', 'expenses.expense')->where('quotation_no', $quotation->quotation_no)->get();
        // dd($quotation);
        $company = Company::where('user_id', auth()->id())->first();
        $tnc = AdminTermsAndCondition::where(['branch_id'=>session()->get('selected_branch'), 'type'=>'quotation'])->first();
        return view('crusher.trans_quotation.view', compact('quotation', 'company', 'tnc'));
    }

    public function edit(Quotation $quotation)
    {
        $customers = Customer::where('branch_id', session()->get('selected_branch'))->get();
        $expenses = Expense::where(['branch_id'=>session()->get('selected_branch'), 'class_of_account_id'=>'Tax & Duties'])
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\Expense');
        });
        $otherAccountExpenses = AdminOtherAccount::where(['branch_id'=>session()->get('selected_branch'), 'class_of_account'=>'Tax & Duties'])
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\AdminOtherAccount');
        });

        $expensesAccount = $expenses->concat($otherAccountExpenses);

        // Retrieve customers and include both 'account_name' and 'table_name' attributes
        $customers = Customer::where('branch_id', session()->get('selected_branch'))
        ->select('*', 'name_of_organisation as account_name')
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\Customer');
        });

        // Retrieve suppliers and include both 'account_name' and 'table_name' attributes
        $suppliers = Supplier::where('branch_id', session()->get('selected_branch'))
        ->select('*', 'name_of_organisation as account_name')
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\Supplier');
        });

        // Retrieve expenses and set 'table_name' to the fully-qualified model class name
                $salesmens = StaffAndSalesman::where('branch_id', session()->get('selected_branch'))
        ->select('*', DB::raw("CONCAT(first_name, ' ', last_name) as account_name"))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\StaffAndSalesman');
        });

        // Retrieve expenses and set 'table_name' to the fully-qualified model class name
        $expenses = Expense::where('branch_id', session()->get('selected_branch'))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\Expense');
        });

        // Retrieve other accounts and set 'table_name' to the fully-qualified model class name
        $otherAccounts = AdminOtherAccount::where('branch_id', session()->get('selected_branch'))
        ->get()
        ->map(function ($item) {
            return $item->setAttribute('table_name', 'App\AdminOtherAccount');
        });

        // Concatenate all collections into a single collection
        $accounts = $customers->concat($expenses)->concat($suppliers)->concat($otherAccounts)->concat($salesmens);


        // $quotations = $quotation->with('account', 'expenses')->where('quotation_no', $quotation->quotation_no)->get();

        $quotations = Quotation::where('branch_id', session()->get('selected_branch'))
        ->with(['account', 'expenses'])
        ->where('quotation_no', $quotation->quotation_no)
        ->get();


        return view('crusher.trans_quotation.edit', compact('accounts', 'quotations', 'expensesAccount'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\Quotation  $quotation
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, Quotation $quotation)
    {
        try {
            $validatedData = $request->validate([
                'quotation_no' => 'nullable|string',
                'date' => 'nullable|date',
                'account_id' => 'nullable',
                'account_type' => 'nullable',
                'item_description' => 'nullable|array',
                'item_description.*' => 'nullable|string',
                'quantity' => 'nullable|array',
                'quantity.*' => 'nullable|numeric',
                'rate_unit' => 'nullable|array',
                'rate_unit.*' => 'nullable|numeric',
                'gross_amount' => 'nullable|array',
                'gross_amount.*' => 'nullable|numeric',
                'vat_percentage' => 'nullable|array',
                'vat_percentage.*' => 'nullable|numeric',
                'vat_amount' => 'nullable|array',
                'vat_amount.*' => 'nullable|numeric',
                'net_amount' => 'nullable|array',
                'net_amount.*' => 'nullable|numeric',
                'expense_type' => 'nullable|array',
                'expense_type.*' => 'nullable',
                'expense_id' => 'nullable|array',
                'expense_id.*' => 'nullable',
                'expense_amount' => 'nullable|array',
                'expense_amount.*' => 'nullable|numeric',
                'total_quantity' => 'nullable|numeric',
                'total_gross_amount' => 'nullable|numeric',
                'total_vat_amount' => 'nullable|numeric',
                'total_net_amount' => 'nullable|numeric',
                'discount' => 'nullable|numeric',

            ]);
            $branch_id = session()->get('selected_branch');

            $quotation->with('account', 'expenses')->where('quotation_no', $quotation->quotation_no)->forceDelete();

            // Iterate over the item descriptions
            for ($i = 0; $i < count($validatedData['quantity']); $i++)
            {
                $quotationData = [
                    'quotation_no' => $validatedData['quotation_no'],
                    'date' => $validatedData['date'],
                    'account_id' => $validatedData['account_id'],
                    'account_type' => $validatedData['account_type'],
                    'item_description' => $validatedData['item_description'][$i]??(NULL),
                    'quantity' => $validatedData['quantity'][$i]??(NULL),
                    'rate_unit' => $validatedData['rate_unit'][$i]??(NULL),
                    'gross_amount' => $validatedData['gross_amount'][$i]??(NULL),
                    'vat_percentage' => $validatedData['vat_percentage'][$i]??(NULL),
                    'vat_amount' => $validatedData['vat_amount'][$i]??(NULL),
                    'net_amount' => $validatedData['net_amount'][$i]??(NULL),
                    'total_quantity' => $validatedData['total_quantity'],
                    'total_gross_amount' => $validatedData['total_gross_amount'],
                    'total_vat_amount' => $validatedData['total_vat_amount'],
                    'total_net_amount' => $validatedData['total_net_amount'],
                    'discount' => $validatedData['discount'],
                    'branch_id' => $branch_id,
                ];

                $quotation = Quotation::create($quotationData);
                QuotationExpense::where('quotation_id', $quotation->id)->forceDelete();
                // Add expenses for this quotation
                // for ($j = 0; $j < count($validatedData['expense_id']); $j++) {
                //     QuotationExpense::create([
                //         'quotation_id' => $quotation->id,
                //         'expense_type' => $validatedData['expense_type'][$j],
                //         'expense_id' => $validatedData['expense_id'][$j],
                //         'expense_amount' => $validatedData['expense_amount'][$j],
                //     ]);
                // }
            }

            flash()->addSuccess('Record Updated');
            return redirect()->route('crusher.quotations.index');
        } catch (\Throwable $th) {
            flash()->addError($th->getMessage());
            return redirect()->back()->withInput();
        }
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\Quotation  $quotation
     * @return \Illuminate\Http\Response
     */
    public function destroy(Quotation $quotation)
    {
        //
    }

    public function softDelete($quotation)
    {
        $quotation = Quotation::findOrFail($quotation);
        $quotation->where('quotation_no', $quotation->quotation_no)->delete();
        flash()->addSuccess('Success');
        return redirect()->route('crusher.quotations.index');
    }
}
