<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Branches;
use App\AdminItemGroup;
use App\ItemName;
use App\AdminSalesInvoice;
use App\AdminPurchaesInvoice;
use App\Customer;
use App\debitNote;
use App\creditNote;
use App\OpeningBalance;

class StockReportController extends Controller
{
    public function view()
    {
        $itemGroups = AdminItemGroup::where('branch_id', session()->get('selected_branch'))->orderBy('id', 'desc')->get();
        $itemNames = ItemName::where('branch_id', session()->get('selected_branch'))->orderBy('id', 'desc')->get();

        return view('crusher.reports.stock_report', compact('itemGroups','itemNames'));

    }
    public function allStockQuery(Request $request)
    {

    $selectedOption = $request->input('ledger_account_class_name');

    if ($selectedOption === 'group_query') {
        $groupId = $request->input('group_stock');

        $itemNames = ItemName::where('branch_id', session()->get('selected_branch'))
            ->where('group_id', $groupId)
            ->orderBy('id', 'desc')->get();

        $groupName = AdminItemGroup::where('branch_id', session()->get('selected_branch'))->where('id', $groupId)->value('name');

        $salesInvoice = AdminSalesInvoice::where('branch_id', session()->get('selected_branch'))
            ->whereIn('item_id', $itemNames->pluck('id'))
            ->whereNotNull('invoice_no')
            ->groupBy('invoice_no')
            ->orderBy('invoice_no')
            ->get();

        $originalPurchaseInvoice = AdminPurchaesInvoice::where('branch_id', session()->get('selected_branch'))
            ->whereIn('item_id', $itemNames->pluck('id'))
            ->whereNotNull('invoice_no')
            // ->groupBy('invoice_no')
            ->orderBy('invoice_no')
            ->get();

            $remainingQuantity = [];
            $remainingValue = [];

            // Create a separate array to track remaining quantities
            $remainingQuantitiesCopy = [];

            foreach ($originalPurchaseInvoice as $purchase) {
                $remainingQuantity[$purchase['id']] = $purchase['quantity'];
                $remainingQuantitiesCopy[$purchase['id']] = &$remainingQuantity[$purchase['id']];
                $remainingValue[$purchase['id']] = $purchase['quantity'] * $purchase['rate_unit'];
            }

            foreach ($salesInvoice as $sale) {
                $qtyToSell = $sale['quantity'];

                foreach ($originalPurchaseInvoice as &$purchase) {
                    $purchaseQty = &$remainingQuantitiesCopy[$purchase['id']];
                    $rateUnit = $purchase['rate_unit'];

                    if ($qtyToSell > 0 && $purchaseQty > 0) {
                        $unitsSold = min($qtyToSell, $purchaseQty);

                        $purchaseQty -= $unitsSold;

                        $remainingValue[$purchase['id']] = $purchaseQty * $rateUnit;

                        $qtyToSell -= $unitsSold;
                    }
                }
            }

            $itemTotals = [];

            foreach ($itemNames as $item) {
                $itemId = $item['id'];
                $totalRemainingQuantity = isset($remainingQuantity[$itemId]) ? $remainingQuantity[$itemId] : 0;
                $totalRemainingValue = isset($remainingValue[$itemId]) ? $remainingValue[$itemId] : 0;
                $averageRate = ($totalRemainingQuantity != 0) ? ($totalRemainingValue / $totalRemainingQuantity) : 0;

                $itemTotals[$itemId] = [
                    'item_id' => $itemId,
                    'item_name' => $item['name'],
                    'total_remaining_quantity' => $totalRemainingQuantity,
                    'total_remaining_value' => $totalRemainingValue,
                    'average_rate' => $averageRate,
                ];
            }

            $itemTotals = array_values($itemTotals); // Re-index the array starting from 0

            // dd($remainingQuantity, $remainingValue, $itemTotals);


        return view('crusher.reports.stock_report_2', compact('itemTotals', 'groupName'));
    }

    elseif ($selectedOption === 'stock_query') {
        return redirect()->route('completeStock');
    }
    elseif ($selectedOption === 'item_query') {
        return redirect()->route('particular.item.query')->withInput();
    }

    }

    public function completeStock(Request $request)
{
    $itemNames = ItemName::where('branch_id', session()->get('selected_branch'))->get();

    $salesInvoice = AdminSalesInvoice::where('branch_id', session()->get('selected_branch'))
        ->whereIn('item_id', $itemNames->pluck('id'))
        ->groupBy('invoice_no')
        ->get();

    $originalPurchaseInvoice = AdminPurchaesInvoice::where('branch_id', session()->get('selected_branch'))
        ->whereIn('item_id', $itemNames->pluck('id'))
        ->whereNotNull('invoice_no')
        ->groupBy('invoice_no')
        ->orderBy('date')
        ->get();

    $salesInvoice = $salesInvoice->toArray();
    $originalPurchaseInvoice = $originalPurchaseInvoice->toArray();

    $remainingQuantity = [];
    $remainingValue = [];

    foreach ($itemNames as $item) {
        $remainingQuantity[$item['id']] = 0;
        $remainingValue[$item['id']] = 0;
    }

    foreach ($originalPurchaseInvoice as $purchase) {
        $itemId = $purchase['item_id'];
        $remainingQuantity[$itemId] += $purchase['quantity'];
        $remainingValue[$itemId] += $purchase['quantity'] * $purchase['rate_unit'];
    }

    foreach ($salesInvoice as $sale) {
        $qtyToSell = $sale['quantity'];
        $itemId = $sale['item_id'];

        foreach ($originalPurchaseInvoice as &$purchase) {
            if ($purchase['item_id'] === $itemId) {
                $purchaseQty = &$remainingQuantity[$itemId];
                $rateUnit = $purchase['rate_unit'];

                if ($qtyToSell > 0 && $purchaseQty > 0) {
                    $unitsSold = min($qtyToSell, $purchaseQty);

                    $purchaseQty -= $unitsSold;

                    // Corrected line, use $purchase['rate_unit'] instead of $rateUnit
                    $remainingValue[$itemId] -= $unitsSold * $purchase['rate_unit'];

                    $qtyToSell -= $unitsSold;
                }
            }
        }
    }

    $itemTotals = [];

    foreach ($itemNames as $item) {
        $itemId = $item['id'];
        $totalRemainingQuantity = $remainingQuantity[$itemId];
        $totalRemainingValue = $remainingValue[$itemId];
        $averageRate = ($totalRemainingQuantity != 0) ? ($totalRemainingValue / $totalRemainingQuantity) : 0;

        $itemTotals[] = [
            'item_id' => $itemId,
            'item_name' => $item['name'],
            'total_remaining_quantity' => $totalRemainingQuantity,
            'total_remaining_value' => $totalRemainingValue,
            'average_rate' => $averageRate,
        ];
    }

    // dd($itemTotals);


    // $itemNames = [
    //     ['id' => 1, 'name' => "item 1", 'description' => "description 1"],
    //     ['id' => 2, 'name' => "item 2", 'description' => "description 2"],
    //     ['id' => 3, 'name' => "item 3", 'description' => "description 3"],
    // ];

    // $originalPurchaseInvoice = [
    //     ['id' => 1, 'item_id' => 1, 'quantity' => 5, 'rate_unit' => 5],
    //     ['id' => 2, 'item_id' => 1, 'quantity' => 2, 'rate_unit' => 4],
    //     ['id' => 3, 'item_id' => 1, 'quantity' => 10, 'rate_unit' => 12],
    //     ['id' => 4, 'item_id' => 2, 'quantity' => 5, 'rate_unit' => 5],
    //     ['id' => 5, 'item_id' => 2, 'quantity' => 2, 'rate_unit' => 4],
    //     ['id' => 6, 'item_id' => 2, 'quantity' => 10, 'rate_unit' => 15],
    //     ['id' => 7, 'item_id' => 3, 'quantity' => 5, 'rate_unit' => 5],
    //     ['id' => 8, 'item_id' => 3, 'quantity' => 2, 'rate_unit' => 4],
    //     ['id' => 9, 'item_id' => 3, 'quantity' => 10, 'rate_unit' => 16],
    // ];

    // $salesInvoice = [
    //     ['id' => 1, 'item_id' => 1, 'quantity' => 3, 'rate_unit' => 10],
    //     ['id' => 2, 'item_id' => 1, 'quantity' => 3, 'rate_unit' => 9],
    //     ['id' => 3, 'item_id' => 2, 'quantity' => 3, 'rate_unit' => 10],
    //     ['id' => 4, 'item_id' => 2, 'quantity' => 3, 'rate_unit' => 8],
    //     ['id' => 5, 'item_id' => 3, 'quantity' => 3, 'rate_unit' => 10],
    //     ['id' => 6, 'item_id' => 3, 'quantity' => 3, 'rate_unit' => 7],
    // ];

    // $remainingQuantity = [];
    // $remainingValue = [];

    // foreach ($itemNames as $item) {
    //     $remainingQuantity[$item['id']] = 0;
    //     $remainingValue[$item['id']] = 0;
    // }

    // foreach ($originalPurchaseInvoice as $purchase) {
    //     $itemId = $purchase['item_id'];
    //     $remainingQuantity[$itemId] += $purchase['quantity'];
    //     $remainingValue[$itemId] += $purchase['quantity'] * $purchase['rate_unit'];
    // }

    // foreach ($salesInvoice as $sale) {
    //     $qtyToSell = $sale['quantity'];
    //     $itemId = $sale['item_id'];

    //     foreach ($originalPurchaseInvoice as &$purchase) {
    //         if ($purchase['item_id'] === $itemId) {
    //             $purchaseQty = &$remainingQuantity[$itemId];
    //             $rateUnit = $purchase['rate_unit'];

    //             if ($qtyToSell > 0 && $purchaseQty > 0) {
    //                 $unitsSold = min($qtyToSell, $purchaseQty);

    //                 $purchaseQty -= $unitsSold;

    //                 // Corrected line, use $purchase['rate_unit'] instead of $rateUnit
    //                 $remainingValue[$itemId] -= $unitsSold * $purchase['rate_unit'];

    //                 $qtyToSell -= $unitsSold;
    //             }
    //         }
    //     }
    // }

    // $itemTotals = [];

    // foreach ($itemNames as $item) {
    //     $itemId = $item['id'];
    //     $totalRemainingQuantity = $remainingQuantity[$itemId];
    //     $totalRemainingValue = $remainingValue[$itemId];
    //     $averageRate = ($totalRemainingQuantity != 0) ? ($totalRemainingValue / $totalRemainingQuantity) : 0;

    //     $itemTotals[] = [
    //         'item_id' => $itemId,
    //         'item_name' => $item['name'],
    //         'total_remaining_quantity' => $totalRemainingQuantity,
    //         'total_remaining_value' => $totalRemainingValue,
    //         'average_rate' => $averageRate,
    //     ];
    // }

    // dd($itemTotals);


    return view('crusher.reports.complete-stock-report', compact('itemTotals'));

}

    // public function completeStock(Request $request)
    // {
    //     $itemNames = ItemName::where('branch_id', session()->get('selected_branch'))->orderBy('id', 'desc')->get();

    //     $PiQTY = AdminPurchaesInvoice::whereIn('item_id', $itemNames->pluck('id')->toArray())
    //         ->pluck('quantity', 'item_id');

    //     $SiQTY = AdminSalesInvoice::where('branch_id', session()->get('selected_branch'))->whereIn('item_id', $itemNames->pluck('id')->toArray())
    //         ->pluck('quantity', 'item_id');
    //     $PrQTY = debitNote::where('branch_id', session()->get('selected_branch'))->whereIn('item_id', $itemNames->pluck('id')->toArray())
    //         ->pluck('quantity', 'item_id');
    //     $SrQTY = creditNote::where('branch_id', session()->get('selected_branch'))->whereIn('item_id', $itemNames->pluck('id')->toArray())
    //         ->pluck('quantity', 'item_id');

    //     $AdminPurchaesInvoiceRates = AdminPurchaesInvoice::where('branch_id', session()->get('selected_branch'))->whereIn('item_id', $itemNames->pluck('id')->toArray())
    //     ->pluck('rate_unit', 'item_id')->toArray();

    //     $AdminSalesInvoiceRates = AdminSalesInvoice::where('branch_id', session()->get('selected_branch'))->whereIn('item_id', $itemNames->pluck('id')->toArray())
    //         ->pluck('rate_unit', 'item_id')->toArray();

    //     $debitNoteRates = debitNote::where('branch_id', session()->get('selected_branch'))->whereIn('item_id', $itemNames->pluck('id')->toArray())
    //         ->pluck('rate_unit', 'item_id')->toArray();

    //     $creditNoteRates = creditNote::where('branch_id', session()->get('selected_branch'))->whereIn('item_id', $itemNames->pluck('id')->toArray())
    //         ->pluck('rate_unit', 'item_id')->toArray();


    //     $quantities = [];
    //     $rates = [];

    //     foreach ($itemNames as $item) {
    //         $purchaseQuantity = $PiQTY[$item->id] ?? 0;
    //         $saleQuantity = $SiQTY[$item->id] ?? 0;
    //         $purchasesReturnQuantity = $PrQTY[$item->id] ?? 0;
    //         $salesReturnQuantity = $SrQTY[$item->id] ?? 0;
    //         $netQuantity = $purchaseQuantity - $saleQuantity - $purchasesReturnQuantity + $salesReturnQuantity;
    //         $quantities[$item->id] = $netQuantity;

    //         $allRates = array_merge(
    //             [$AdminPurchaesInvoiceRates[$item->id] ?? null],
    //             [$AdminSalesInvoiceRates[$item->id] ?? null],
    //             [$debitNoteRates[$item->id] ?? null],
    //             [$creditNoteRates[$item->id] ?? null]
    //         );

    //         $filteredRates = array_filter($allRates, fn ($rate) => $rate !== null);
    //         $averageRate = empty($filteredRates) ? null : array_sum($filteredRates) / count($filteredRates);
    //         $rates[$item->id] = $averageRate;
    //     }

    //     // dd($quantities,$rates);

    //     return view('crusher.reports.complete-stock-report', compact('itemNames', 'quantities','rates'));
    // }

    public function particularItemQuery(Request $request)
    {
        $from_date = $request->old('from_date');
        $to_date = $request->old('to_date');
        $item_stock = $request->old('item_stock');
        $currentYearFirstDate = date("2003-01-01");
        $oneDayPreviousDate = date('Y-m-d', strtotime('-1 day', strtotime($from_date)));
        $item = ItemName::find($item_stock);

        if (!$item) {
            abort(404);
        }

        $salesInvoice = AdminSalesInvoice::with('customer','supplier','StaffAndSalesman','Expense','AdminOtherAccount')
            ->where('branch_id', session()->get('selected_branch'))
            ->where('item_id', $item_stock)
            ->where('date', '>=', $from_date . ' 00:00:00')
            ->where('date', '<=', $to_date . ' 23:59:59')
            // ->groupBy('invoice_no')
            ->orderBy('invoice_no')
            ->get();

        $originalPurchaseInvoice = AdminPurchaesInvoice::with('customer','supplier','StaffAndSalesman','Expense','AdminOtherAccount')
            ->where('branch_id', session()->get('selected_branch'))
            ->where('item_id', $item_stock)
            ->where('date', '>=', $from_date . ' 00:00:00')
            ->where('date', '<=', $to_date . ' 23:59:59')
            ->whereNotNull('invoice_no')
            // ->groupBy('invoice_no')
            ->orderBy('invoice_no')
            ->get();
        $openingBalanceForInventory = OpeningBalance::with('customer','supplier','StaffAndSalesman','Expense','AdminOtherAccount')
            ->where('branch_id', session()->get('selected_branch'))
            ->where('item_id', $item_stock)
            ->where('date', '>=', $from_date . ' 00:00:00')
            ->where('date', '<=', $to_date . ' 23:59:59')
            ->whereNotNull('invoice_no')
            // ->groupBy('invoice_no')
            ->orderBy('invoice_no')
            ->get();
        $obSalesInvoice = AdminSalesInvoice::with('customer','supplier','StaffAndSalesman','Expense','AdminOtherAccount')
        ->where('branch_id', session()->get('selected_branch'))
        ->where('item_id', $item_stock)
        ->where('date', '>=', $currentYearFirstDate . ' 00:00:00')
        ->where('date', '<=', $oneDayPreviousDate . ' 23:59:59')
        // ->groupBy('invoice_no')
        ->orderBy('invoice_no')
        ->get();
         $obPurchaseInvoice = AdminPurchaesInvoice::with('customer', 'supplier', 'StaffAndSalesman', 'Expense', 'AdminOtherAccount')
         ->where('branch_id', session()->get('selected_branch'))
         ->where('item_id', $item_stock)
         ->where('date', '>=', $currentYearFirstDate . ' 00:00:00')
         ->where('date', '<=', $oneDayPreviousDate . ' 23:59:59')
         ->whereNotNull('invoice_no')
         ->orderBy('invoice_no')
         ->get();
        $cbSalesInvoice = AdminSalesInvoice::with('customer','supplier','StaffAndSalesman','Expense','AdminOtherAccount')
        ->where('branch_id', session()->get('selected_branch'))
        ->where('item_id', $item_stock)
        ->where('date', '>=', $currentYearFirstDate . ' 00:00:00')
        ->where('date', '<=', $from_date . ' 23:59:59')
        // ->groupBy('invoice_no')
        ->orderBy('invoice_no')
        ->get();
         $cbPurchaseInvoice = AdminPurchaesInvoice::with('customer', 'supplier', 'StaffAndSalesman', 'Expense', 'AdminOtherAccount')
         ->where('branch_id', session()->get('selected_branch'))
         ->where('item_id', $item_stock)
         ->where('date', '>=', $currentYearFirstDate . ' 00:00:00')
         ->where('date', '<=', $oneDayPreviousDate . ' 23:59:59')
         ->whereNotNull('invoice_no')
         ->orderBy('invoice_no')
         ->get();

        $salesInvoice = $salesInvoice->toArray();
        $openingBalanceForInventory = $openingBalanceForInventory->toArray();
        $originalPurchaseInvoice = $originalPurchaseInvoice->toArray();

        $obSalesInvoice = $obSalesInvoice->toArray();
        $obPurchaseInvoice = $obPurchaseInvoice->toArray();

        $cbSalesInvoice = $cbSalesInvoice->toArray();
        $cbPurchaseInvoice = $cbPurchaseInvoice->toArray();



        $profits = [];
        $remainingPurchaseQty = [];
        $remainingPurchaseRate = [];
        $obRemainingPurchaseQty = [];
        $obRemainingPurchaseRate = [];
        $cbRemainingPurchaseQty = [];
        $cbRemainingPurchaseRate = [];

        $cbPurchaseInvoices = $cbPurchaseInvoice;

        foreach ($cbSalesInvoice as $key => &$sale) {
            $qtyToSell = $sale['quantity'];
            $rate = $sale['rate_unit'];
            $profit = 0;

            foreach ($cbPurchaseInvoices as $purchaseKey => &$purchase) {
                $purchaseQty = $purchase['quantity'];

                if ($qtyToSell > 0 && $purchaseQty > 0) {
                    $unitsSold = min($qtyToSell, $purchaseQty);

                    $profit += $unitsSold * ($rate - $purchase['rate_unit']);

                    $cbPurchaseInvoices[$purchaseKey]['quantity'] -= $unitsSold;

                    $qtyToSell -= $unitsSold;
                }
            }

            unset($purchase);

            $profits[$sale['id']] = $profit;
            $sale['profit'] = $profit;

        }
        $obPurchaseInvoices = $obPurchaseInvoice;

        foreach ($obSalesInvoice as $key => &$sale) {
            $qtyToSell = $sale['quantity'];
            $rate = $sale['rate_unit'];
            $profit = 0;

            foreach ($obPurchaseInvoices as $purchaseKey => &$purchase) {
                $purchaseQty = $purchase['quantity'];

                if ($qtyToSell > 0 && $purchaseQty > 0) {
                    $unitsSold = min($qtyToSell, $purchaseQty);

                    $profit += $unitsSold * ($rate - $purchase['rate_unit']);

                    $obPurchaseInvoices[$purchaseKey]['quantity'] -= $unitsSold;

                    $qtyToSell -= $unitsSold;
                }
            }

            unset($purchase);

            $profits[$sale['id']] = $profit;
            $sale['profit'] = $profit;

        }

        // $purchaseInvoice = $openingBalanceForInventory + $originalPurchaseInvoice;
        $purchaseInvoice = array_merge($openingBalanceForInventory, $originalPurchaseInvoice);

        foreach ($salesInvoice as $key => &$sale) {
            $qtyToSell = $sale['quantity'];
            $rate = $sale['rate_unit'];
            $profit = 0;

            foreach ($purchaseInvoice as $purchaseKey => &$purchase) {
                $purchaseQty = $purchase['quantity'];

                if ($qtyToSell > 0 && $purchaseQty > 0) {
                    $unitsSold = min($qtyToSell, $purchaseQty);

                    $profit += $unitsSold * ($rate - $purchase['rate_unit']);

                    $purchaseInvoice[$purchaseKey]['quantity'] -= $unitsSold;

                    $qtyToSell -= $unitsSold;
                }
            }

            unset($purchase);

            $profits[$sale['id']] = $profit;
            $sale['profit'] = $profit;

        }

        foreach ($purchaseInvoice as $purchaseKey => $remainingPurchase) {
            $remainingPurchaseQty[$purchaseInvoice[$purchaseKey]['id']] = $remainingPurchase['quantity'];
            $remainingPurchaseRate[$purchaseInvoice[$purchaseKey]['id']] = $remainingPurchase['quantity'] * $remainingPurchase['rate_unit'];

        }

        foreach ($obPurchaseInvoices as $purchaseKey => $remainingPurchase) {
            $obRemainingPurchaseQty[$obPurchaseInvoices[$purchaseKey]['id']] = $remainingPurchase['quantity'];
            $obRemainingPurchaseRate[$obPurchaseInvoices[$purchaseKey]['id']] = $remainingPurchase['quantity'] * $remainingPurchase['rate_unit'];
        }

        foreach ($cbPurchaseInvoices as $purchaseKey => $remainingPurchase) {
            $cbRemainingPurchaseQty[$cbPurchaseInvoices[$purchaseKey]['id']] = $remainingPurchase['quantity'];
            $cbRemainingPurchaseRate[$cbPurchaseInvoices[$purchaseKey]['id']] = $remainingPurchase['quantity'] * $remainingPurchase['rate_unit'];
        }

        $totalRemainingPurchaseQty = array_sum($remainingPurchaseQty);
        $totalRemainingPurchaseRate = array_sum($remainingPurchaseRate);
        $totalObRemainingPurchaseQty = array_sum($obRemainingPurchaseQty);
        $totalObRemainingPurchaseRate = array_sum($obRemainingPurchaseRate);
        $totalcbRemainingPurchaseQty = array_sum($cbRemainingPurchaseQty);
        $totalcbRemainingPurchaseRate = array_sum($cbRemainingPurchaseRate);

        $concatenatedData = array_merge($openingBalanceForInventory, $originalPurchaseInvoice, $salesInvoice);

        // dd($concatenatedData,$totalRemainingPurchaseQty,$totalRemainingPurchaseRate);
        // dd($concatenatedData,$remainingPurchaseQty,$remainingPurchaseRate,$totalRemainingPurchaseQty,$totalRemainingPurchaseRate,$totalObRemainingPurchaseQty,$totalObRemainingPurchaseRate);

        return view('crusher.reports.particular-item-query', compact('item', 'salesInvoice','openingBalanceForInventory','originalPurchaseInvoice', 'from_date', 'to_date', 'item_stock', 'concatenatedData','profits','totalRemainingPurchaseQty','totalRemainingPurchaseRate','totalObRemainingPurchaseQty','totalObRemainingPurchaseRate','totalcbRemainingPurchaseQty','totalcbRemainingPurchaseRate'));
    }

}
